//==========================================================================================
//            eMProBaCA 1.0 - mp3PRO Batch Conversion Assistant by Efenstor
//                    Copyright (C) 2003 Andrey "Efenstor" Pivovarov
//------------------------------------------------------------------------------------------
// NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED. YOU USE THE PROGRAM AT YOUR OWN RISK.
// THE AUTHOR DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING THE WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. NOBODY WILL BE LIABLE FOR DATA
// LOSS, DAMAGES, LOSS OF PROFITS OR ANY OTHER KIND OF LOSS WHILE USING OR MISUSING THIS
// SOFTWARE.
//------------------------------------------------------------------------------------------
// This program is free to use and distribute. The only thing is prohibited is to
// make money with it. It is free and MUST remain free forever.
//------------------------------------------------------------------------------------------
// Do not contact the author with meaningless and ridiculous problems, write me only if
// you've surely found a bug.
//                                                  Sincerely thy, forever and aye
//                                                                        Megester Efenstor
//                                                         E-mail: efenstor@stratopoint.com
//==========================================================================================


#include "stdafx.h"
#include <stdlib.h>
#include <shellapi.h>
#include "resource.h"


#define INIFILENAME "eMProBaCA.ini"
#define MID_HELP 0
#define MID_PLAYERNOTFOUND 1
#define MID_CANNOTLAUNCH 2
#define MID_CANNOTDELETEOLD 3
#define MID_CANNOTLOADUSER32 4


HINSTANCE g_hInstance;

CHAR g_iniFilePath[_MAX_PATH+1], g_playerPath[_MAX_PATH+1], g_srcPath[_MAX_PATH+1],
	g_destPath[_MAX_PATH+1];


//==========================================================================================
// SplitFilename
//==========================================================================================
VOID SplitFilename(CHAR *fullPath, CHAR *path, CHAR *namewithExt, CHAR *name, CHAR *ext)
{
	// Let's try to find wrong characters
	//if(strchr(fullPath, '?') || strchr(fullPath, '*'))
	//{
	//	if(path) strcpy(path, "");
	//	if(namewithExt) strcpy(namewithExt, "");
	//	if(name) strcpy(name, "");
	//	if(ext) strcpy(ext, "");
	//}

	// Find a last slash and a last dot
	CHAR *lastSlash = strrchr(fullPath, '\\');
	if(!lastSlash) lastSlash = strrchr(fullPath, '/');
	CHAR *dot;
	if(!lastSlash)
		// If there is no path
		dot = strrchr(fullPath, '.');
	else
		dot = strrchr(lastSlash, '.');

	UINT len;
	// Extract extension
	if(!dot || lastSlash>dot)
	{
		// If there is no extension
		if(lastSlash) len = (UINT)strlen(lastSlash+1);
		else len = (UINT)strlen(fullPath);
		if(ext) strcpy(ext, "");
	} else {
		if(lastSlash) len = (UINT)(dot - lastSlash - 1);
		else len = (UINT)(dot - fullPath);
		if(ext) strcpy(ext, dot+1);
	}

	// Extract path
	if(path)
		if(!lastSlash)
			// If there is no path
			strcpy(path, "");
		else {
			strncpy(path, fullPath, lastSlash-fullPath);
			if(path[lastSlash-fullPath-1]==':')
			{
				path[lastSlash-fullPath] = '\\';
				path[lastSlash-fullPath+1] = 0;
			}
			else path[lastSlash-fullPath] = 0;
		}

	// Extract name
	if(name)
		if(!lastSlash)
		{
			// If there is no path
			strncpy(name, fullPath, len);
			name[len] = 0;
		} else {
			strncpy(name, lastSlash+1, len);
			name[len] = 0;
		}

	// Extract name with extension
	if(namewithExt)
		if(!lastSlash)
			// If there is no path
			strcpy(namewithExt, fullPath);
		else
			strcpy(namewithExt, lastSlash+1);
}



//==========================================================================================
// SkipSpaces - skips spaces, CR's and TAB's
//==========================================================================================
CHAR* SkipSpaces(CHAR *buffer)
{
	while(buffer[0]==' ' || buffer[0]==0x0D || buffer[0]==0x0A || buffer[0]=='\t')
		buffer++;
	return buffer;
}


//==========================================================================================
// CopyString - copies until "stopChar" or NULL is met
// return: length
//==========================================================================================
INT CopyString(CHAR *dest, CHAR *src, CHAR stopChar)
{
	for(UINT pos = 0; src[pos]!=stopChar && src[pos]!=0; pos++)
		dest[pos] = src[pos];
	return pos;
}


//==========================================================================================
// HelpDlgProc
//==========================================================================================
BOOL CALLBACK HelpDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_INITDIALOG:
		// Set the icon
		SetClassLong(hwndDlg, GCL_HICON, (long)LoadIcon(g_hInstance, MAKEINTRESOURCE(
			IDI_APPICO)));
		break;

	case WM_COMMAND:
		// Process items
		switch(LOWORD(wParam))
		{
		case IDOK:
		case IDCANCEL:
			PostQuitMessage(0);
			break;
		}
		break;

	case WM_CLOSE:
		PostQuitMessage(0);
		break;

	default:
		// Everything else
		return 0;
	}

	return -1;
}


//==========================================================================================
// BlockedDlgProc
//==========================================================================================
BOOL CALLBACK BlockedDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return -1;
}


//==========================================================================================
// MessageAndQuit - displays message (help / error) and quits
//==========================================================================================
VOID MessageAndQuit(UINT messageId)
{
	switch(messageId)
	{
	case MID_HELP:
		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_HELP), NULL, HelpDlgProc);
		break;

	case MID_PLAYERNOTFOUND:
		MessageBox(NULL, "THOMSON mp3PRO Audio Player was not found in the specified location!",
			"eMProBaCA", MB_OK|MB_ICONERROR);
		break;

	case MID_CANNOTLAUNCH:
		MessageBox(NULL, "Cannot launch THOMSON mp3PRO Audio Player!",
			"eMProBaCA", MB_OK|MB_ICONERROR);
		break;

	case MID_CANNOTDELETEOLD:
		MessageBox(NULL, "Cannot delete the old file with the same name as the destination!",
			"eMProBaCA", MB_OK|MB_ICONERROR);
		break;

	case MID_CANNOTLOADUSER32:
		MessageBox(NULL, "Cannot load \"User32.dll\"!",
			"eMProBaCA", MB_OK|MB_ICONERROR);
		break;
	}

	exit(0);
}


//==========================================================================================
// WinMain
//------------------------------------------------------------------------------------------
// cmdline: eMProBaCA.exe <path to mp3PROplayer.exe.exe> <path to the file to encode>
//          <path to the output file>
//==========================================================================================
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	// Load "BlockInput"
	HMODULE hUser32Dll = LoadLibrary("User32.dll");
	if(!hUser32Dll) MessageAndQuit(MID_CANNOTLOADUSER32);
	BOOL(WINAPI *hBlockInput)(BOOL) = (BOOL(WINAPI*)(BOOL))GetProcAddress(hUser32Dll,
		"BlockInput");

	// Set some variables
	g_hInstance = hInstance;
	CHAR path[_MAX_PATH+1];
	GetModuleFileName(g_hInstance, path, _MAX_PATH+1);
	SplitFilename(path, g_iniFilePath, NULL, NULL, NULL);
	if(g_iniFilePath[strlen(g_iniFilePath)-1] != '\\') strcat(g_iniFilePath, "\\");
	strcat(g_iniFilePath, INIFILENAME);

	// Should we use the safe mode?
	BOOL safeMode = FALSE;
	if(GetPrivateProfileInt("General", "SafeMode", 0, g_iniFilePath))
		safeMode = TRUE;

	// Get paths
	CHAR *clPos = SkipSpaces(lpCmdLine);
	if(clPos[0]==0) MessageAndQuit(MID_HELP);
	if(clPos[0]=='\"') {
		clPos += CopyString(g_playerPath, clPos+1, '\"');
		clPos+=2;
	} else clPos += CopyString(g_playerPath, clPos, ' ');

	clPos = SkipSpaces(clPos);
	if(clPos[0]==0) MessageAndQuit(MID_HELP);
	if(clPos[0]=='\"') {
		clPos += CopyString(g_srcPath, clPos+1, '\"');
		clPos+=2;
	} else clPos += CopyString(g_srcPath, clPos, ' ');

	clPos = SkipSpaces(clPos);
	if(clPos[0]==0) MessageAndQuit(MID_HELP);
	if(clPos[0]=='\"')
		CopyString(g_destPath, clPos+1, '\"');
	else CopyString(g_destPath, clPos, ' ');

	// Check if the destination file already exists then delete it (it's compulsory!)
	WIN32_FIND_DATA fd;
	HANDLE h;
	h = FindFirstFile(g_destPath, &fd);
	if(h!=INVALID_HANDLE_VALUE)
	{
		FindClose(h);
		if(!DeleteFile(g_destPath)) MessageAndQuit(MID_CANNOTDELETEOLD);
	}

	// Block keyboard and mouse
	if(hBlockInput && !safeMode) hBlockInput(TRUE);

	// Get current foreground window
	HWND hOldForeground = GetForegroundWindow();

	// Execute the player
	SHELLEXECUTEINFO sei;
	ZeroMemory(&sei, sizeof(sei));
	sei.cbSize = sizeof(sei);
	sei.fMask = SEE_MASK_FLAG_NO_UI|SEE_MASK_NOCLOSEPROCESS;
	sei.lpVerb = "open";
	sei.lpFile = g_playerPath;
	sei.nShow = SW_SHOW;
	if(!ShellExecuteEx(&sei)) MessageAndQuit(MID_PLAYERNOTFOUND);
	if(sei.hProcess==NULL) MessageAndQuit(MID_CANNOTLAUNCH);

	// Wait for the main window
	HWND hPlayerWnd;
	DWORD status;
	do {
		// Check if the player still here
		GetExitCodeProcess(sei.hProcess, &status);
		if(status!=STILL_ACTIVE)
		{
			// Unblock keyboard and mouse
			if(hBlockInput && !safeMode) hBlockInput(FALSE);
			return 0;
		}
		// Try to find the window
		hPlayerWnd = FindWindow("#32770", "THOMSON mp3PRO Audio Player");
		if(safeMode) Sleep(100);
	} while (!hPlayerWnd);

	// Bring the old foreground window up (to prevent possible user's interference)
	if(!hBlockInput || safeMode) SetForegroundWindow(hOldForeground);

	// Find the "Encode" button and kinda click it
	HWND hEncodeBtn = FindWindowEx(hPlayerWnd, NULL, "Button", "encode");
	PostMessage(hEncodeBtn, BM_CLICK, 0, 0);

	// Wait for the encoder window
	HWND hEncoderWnd = NULL;
	do {
		// Check if the player still here
		GetExitCodeProcess(sei.hProcess, &status);
		if(status!=STILL_ACTIVE)
		{
			// Unblock keyboard and mouse
			if(hBlockInput && !safeMode) hBlockInput(FALSE);
			return 0;
		}
		// Try to find the window
		hEncoderWnd = FindWindowEx(NULL, hEncoderWnd, "#32770", "THOMSON mp3PRO Encoder");
		if(safeMode) Sleep(100);
	} while (GetParent(hEncoderWnd)!=hPlayerWnd);

	// Bring the old foreground window up (to prevent possible user's interference)
	if(!hBlockInput || safeMode) SetForegroundWindow(hOldForeground);

	// Find controls
	HWND hEncodeBtn2 = FindWindowEx(hEncoderWnd, NULL, "Button", "ENCODE");
	HWND hInputFld = FindWindowEx(hEncoderWnd, NULL, "Edit", NULL);
	HWND hOutputFld = FindWindowEx(hEncoderWnd, hInputFld, "Edit", NULL);
	HWND hIStereoCB = FindWindowEx(hEncoderWnd, NULL, "Button", "Allow Intensity Stereo");
	HWND hExtraHQCB = FindWindowEx(hEncoderWnd, NULL, "Button", "Extra High Quality");
	HWND hCodeOFLCB = FindWindowEx(hEncoderWnd, NULL, "Button", "Code Original File Length");

	// Set texts
	SendMessage(hInputFld, WM_SETTEXT, 0, (LPARAM)g_srcPath);
	SendMessage(hOutputFld, WM_SETTEXT, 0, (LPARAM)g_destPath);

	// Check checkboxes
	if(GetPrivateProfileInt("Options", "AllowIntensityStereo", 0, g_iniFilePath))
		SendMessage(hIStereoCB, BM_SETCHECK, BST_CHECKED, 0);
	else SendMessage(hIStereoCB, BM_SETCHECK, BST_UNCHECKED, 0);
	if(GetPrivateProfileInt("Options", "ExtraHighQuality", 1, g_iniFilePath))
		SendMessage(hExtraHQCB, BM_SETCHECK, BST_CHECKED, 0);
	else SendMessage(hExtraHQCB, BM_SETCHECK, BST_UNCHECKED, 0);
	if(GetPrivateProfileInt("Options", "CodeOriginalFileLength", 1, g_iniFilePath))
		SendMessage(hCodeOFLCB, BM_SETCHECK, BST_CHECKED, 0);
	else SendMessage(hCodeOFLCB, BM_SETCHECK, BST_UNCHECKED, 0);

	// Sorta click "ENCODE"
	PostMessage(hEncodeBtn2, BM_CLICK, 0, 0);

	// Lower player's priority class
	if(GetPrivateProfileInt("General", "LowerPlayerPriority", 1, g_iniFilePath))
		SetPriorityClass(sei.hProcess, IDLE_PRIORITY_CLASS);

	// Minimize the player
	if(GetPrivateProfileInt("General", "MinimizePlayer", 1, g_iniFilePath))
	{
		// Send to back
		SetWindowPos(hPlayerWnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOACTIVATE|SWP_NOMOVE|
			SWP_NOSIZE);
		// Minimize
		CloseWindow(hPlayerWnd);
	}

	// Bring the old foreground window up
	SetForegroundWindow(hOldForeground);

	// Unblock keyboard and mouse
	if(hBlockInput && !safeMode) hBlockInput(FALSE);

	// Wait until everything's finished (or Player window is closed (if we pressed "Stop"))
	do {
		// Check if the player still here
		GetExitCodeProcess(sei.hProcess, &status);
		if(status!=STILL_ACTIVE) return 0;
		// Try to open the file
		h = CreateFile(g_destPath, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
		Sleep(safeMode?1000:500);
	} while (h==INVALID_HANDLE_VALUE);
	CloseHandle(h);

	// Close the player
	TerminateProcess(sei.hProcess, 0);

	// Wait until it's terminated
	while(1)
	{
		GetExitCodeProcess(sei.hProcess, &status);
		if(status!=STILL_ACTIVE) break;
		if(safeMode) Sleep(100);
	}

	// Delete the source file
	if(GetPrivateProfileInt("General", "DeleteSourceFile", 0, g_iniFilePath))
		DeleteFile(g_srcPath);
	return 0;
}
